// node_modules/.pnpm/ass-compiler@0.1.14/node_modules/ass-compiler/dist/esm/ass-compiler.js
function parseEffect(text) {
  var param = text.toLowerCase().trim().split(/\s*;\s*/);
  if (param[0] === "banner") {
    return {
      name: param[0],
      delay: param[1] * 1 || 0,
      leftToRight: param[2] * 1 || 0,
      fadeAwayWidth: param[3] * 1 || 0
    };
  }
  if (/^scroll\s/.test(param[0])) {
    return {
      name: param[0],
      y1: Math.min(param[1] * 1, param[2] * 1),
      y2: Math.max(param[1] * 1, param[2] * 1),
      delay: param[3] * 1 || 0,
      fadeAwayHeight: param[4] * 1 || 0
    };
  }
  if (text !== "") {
    return { name: text };
  }
  return null;
}
function parseDrawing(text) {
  if (!text) {
    return [];
  }
  return text.toLowerCase().replace(/([+-]?(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?)/g, " $1 ").replace(/([mnlbspc])/g, " $1 ").trim().replace(/\s+/g, " ").split(/\s(?=[mnlbspc])/).map(function(cmd) {
    return cmd.split(" ").filter(function(x, i) {
      return !(i && isNaN(x * 1));
    });
  });
}
var numTags = [
  "b",
  "i",
  "u",
  "s",
  "fsp",
  "k",
  "K",
  "kf",
  "ko",
  "kt",
  "fe",
  "q",
  "p",
  "pbo",
  "a",
  "an",
  "fscx",
  "fscy",
  "fax",
  "fay",
  "frx",
  "fry",
  "frz",
  "fr",
  "be",
  "blur",
  "bord",
  "xbord",
  "ybord",
  "shad",
  "xshad",
  "yshad"
];
var numRegexs = numTags.map(function(nt) {
  return { name: nt, regex: new RegExp("^" + nt + "-?\\d") };
});
function parseTag(text) {
  var assign;
  var tag = {};
  for (var i = 0; i < numRegexs.length; i++) {
    var ref = numRegexs[i];
    var name = ref.name;
    var regex = ref.regex;
    if (regex.test(text)) {
      tag[name] = text.slice(name.length) * 1;
      return tag;
    }
  }
  if (/^fn/.test(text)) {
    tag.fn = text.slice(2);
  } else if (/^r/.test(text)) {
    tag.r = text.slice(1);
  } else if (/^fs[\d+-]/.test(text)) {
    tag.fs = text.slice(2);
  } else if (/^\d?c&?H?[0-9a-fA-F]+|^\d?c$/.test(text)) {
    var ref$1 = text.match(/^(\d?)c&?H?(\w*)/);
    var num = ref$1[1];
    var color = ref$1[2];
    tag["c" + (num || 1)] = color && ("000000" + color).slice(-6);
  } else if (/^\da&?H?[0-9a-fA-F]+/.test(text)) {
    var ref$2 = text.match(/^(\d)a&?H?([0-9a-f]+)/i);
    var num$1 = ref$2[1];
    var alpha = ref$2[2];
    tag["a" + num$1] = ("00" + alpha).slice(-2);
  } else if (/^alpha&?H?[0-9a-fA-F]+/.test(text)) {
    assign = text.match(/^alpha&?H?([0-9a-f]+)/i), tag.alpha = assign[1];
    tag.alpha = ("00" + tag.alpha).slice(-2);
  } else if (/^(?:pos|org|move|fad|fade)\([^)]+/.test(text)) {
    var ref$3 = text.match(/^(\w+)\((.*?)\)?$/);
    var key = ref$3[1];
    var value = ref$3[2];
    tag[key] = value.trim().split(/\s*,\s*/).map(Number);
  } else if (/^i?clip\([^)]+/.test(text)) {
    var p = text.match(/^i?clip\((.*?)\)?$/)[1].trim().split(/\s*,\s*/);
    tag.clip = {
      inverse: /iclip/.test(text),
      scale: 1,
      drawing: null,
      dots: null
    };
    if (p.length === 1) {
      tag.clip.drawing = parseDrawing(p[0]);
    }
    if (p.length === 2) {
      tag.clip.scale = p[0] * 1;
      tag.clip.drawing = parseDrawing(p[1]);
    }
    if (p.length === 4) {
      tag.clip.dots = p.map(Number);
    }
  } else if (/^t\(/.test(text)) {
    var p$1 = text.match(/^t\((.*?)\)?$/)[1].trim().replace(/\\.*/, function(x) {
      return x.replace(/,/g, "\n");
    }).split(/\s*,\s*/);
    if (!p$1[0]) {
      return tag;
    }
    tag.t = {
      t1: 0,
      t2: 0,
      accel: 1,
      tags: p$1[p$1.length - 1].replace(/\n/g, ",").split("\\").slice(1).map(parseTag)
    };
    if (p$1.length === 2) {
      tag.t.accel = p$1[0] * 1;
    }
    if (p$1.length === 3) {
      tag.t.t1 = p$1[0] * 1;
      tag.t.t2 = p$1[1] * 1;
    }
    if (p$1.length === 4) {
      tag.t.t1 = p$1[0] * 1;
      tag.t.t2 = p$1[1] * 1;
      tag.t.accel = p$1[2] * 1;
    }
  }
  return tag;
}
function parseTags(text) {
  var tags = [];
  var depth = 0;
  var str = "";
  var transText = text.split("\\").slice(1).concat("").join("\\");
  for (var i = 0; i < transText.length; i++) {
    var x = transText[i];
    if (x === "(") {
      depth++;
    }
    if (x === ")") {
      depth--;
    }
    if (depth < 0) {
      depth = 0;
    }
    if (!depth && x === "\\") {
      if (str) {
        tags.push(str);
      }
      str = "";
    } else {
      str += x;
    }
  }
  return tags.map(parseTag);
}
function parseText(text) {
  var pairs = text.split(/{([^{}]*?)}/);
  var parsed = [];
  if (pairs[0].length) {
    parsed.push({ tags: [], text: pairs[0], drawing: [] });
  }
  for (var i = 1; i < pairs.length; i += 2) {
    var tags = parseTags(pairs[i]);
    var isDrawing = tags.reduce(function(v, tag) {
      return tag.p === void 0 ? v : !!tag.p;
    }, false);
    parsed.push({
      tags,
      text: isDrawing ? "" : pairs[i + 1],
      drawing: isDrawing ? parseDrawing(pairs[i + 1]) : []
    });
  }
  return {
    raw: text,
    combined: parsed.map(function(frag) {
      return frag.text;
    }).join(""),
    parsed
  };
}
function parseTime(time) {
  var t = time.split(":");
  return t[0] * 3600 + t[1] * 60 + t[2] * 1;
}
function parseDialogue(text, format) {
  var fields = text.split(",");
  if (fields.length > format.length) {
    var textField = fields.slice(format.length - 1).join();
    fields = fields.slice(0, format.length - 1);
    fields.push(textField);
  }
  var dia = {};
  for (var i = 0; i < fields.length; i++) {
    var fmt = format[i];
    var fld = fields[i].trim();
    switch (fmt) {
      case "Layer":
      case "MarginL":
      case "MarginR":
      case "MarginV":
        dia[fmt] = fld * 1;
        break;
      case "Start":
      case "End":
        dia[fmt] = parseTime(fld);
        break;
      case "Effect":
        dia[fmt] = parseEffect(fld);
        break;
      case "Text":
        dia[fmt] = parseText(fld);
        break;
      default:
        dia[fmt] = fld;
    }
  }
  return dia;
}
var stylesFormat = ["Name", "Fontname", "Fontsize", "PrimaryColour", "SecondaryColour", "OutlineColour", "BackColour", "Bold", "Italic", "Underline", "StrikeOut", "ScaleX", "ScaleY", "Spacing", "Angle", "BorderStyle", "Outline", "Shadow", "Alignment", "MarginL", "MarginR", "MarginV", "Encoding"];
var eventsFormat = ["Layer", "Start", "End", "Style", "Name", "MarginL", "MarginR", "MarginV", "Effect", "Text"];
function parseFormat(text) {
  var fields = stylesFormat.concat(eventsFormat);
  return text.match(/Format\s*:\s*(.*)/i)[1].split(/\s*,\s*/).map(function(field) {
    var caseField = fields.find(function(f) {
      return f.toLowerCase() === field.toLowerCase();
    });
    return caseField || field;
  });
}
function parseStyle(text, format) {
  var values = text.match(/Style\s*:\s*(.*)/i)[1].split(/\s*,\s*/);
  return Object.assign.apply(Object, [{}].concat(format.map(function(fmt, idx) {
    var obj;
    return obj = {}, obj[fmt] = values[idx], obj;
  })));
}
function parse(text) {
  var tree = {
    info: {},
    styles: { format: [], style: [] },
    events: { format: [], comment: [], dialogue: [] }
  };
  var lines = text.split(/\r?\n/);
  var state = 0;
  for (var i = 0; i < lines.length; i++) {
    var line = lines[i].trim();
    if (/^;/.test(line)) {
      continue;
    }
    if (/^\[Script Info\]/i.test(line)) {
      state = 1;
    } else if (/^\[V4\+? Styles\]/i.test(line)) {
      state = 2;
    } else if (/^\[Events\]/i.test(line)) {
      state = 3;
    } else if (/^\[.*\]/.test(line)) {
      state = 0;
    }
    if (state === 0) {
      continue;
    }
    if (state === 1) {
      if (/:/.test(line)) {
        var ref = line.match(/(.*?)\s*:\s*(.*)/);
        var key = ref[1];
        var value = ref[2];
        tree.info[key] = value;
      }
    }
    if (state === 2) {
      if (/^Format\s*:/i.test(line)) {
        tree.styles.format = parseFormat(line);
      }
      if (/^Style\s*:/i.test(line)) {
        tree.styles.style.push(parseStyle(line, tree.styles.format));
      }
    }
    if (state === 3) {
      if (/^Format\s*:/i.test(line)) {
        tree.events.format = parseFormat(line);
      }
      if (/^(?:Comment|Dialogue)\s*:/i.test(line)) {
        var ref$1 = line.match(/^(\w+?)\s*:\s*(.*)/i);
        var key$1 = ref$1[1];
        var value$1 = ref$1[2];
        tree.events[key$1.toLowerCase()].push(parseDialogue(value$1, tree.events.format));
      }
    }
  }
  return tree;
}
function createCommand(arr) {
  var cmd = {
    type: null,
    prev: null,
    next: null,
    points: []
  };
  if (/[mnlbs]/.test(arr[0])) {
    cmd.type = arr[0].toUpperCase().replace("N", "L").replace("B", "C");
  }
  for (var len = arr.length - !(arr.length & 1), i = 1; i < len; i += 2) {
    cmd.points.push({ x: arr[i] * 1, y: arr[i + 1] * 1 });
  }
  return cmd;
}
function isValid(cmd) {
  if (!cmd.points.length || !cmd.type) {
    return false;
  }
  if (/C|S/.test(cmd.type) && cmd.points.length < 3) {
    return false;
  }
  return true;
}
function getViewBox(commands) {
  var ref;
  var minX = Infinity;
  var minY = Infinity;
  var maxX = -Infinity;
  var maxY = -Infinity;
  (ref = []).concat.apply(ref, commands.map(function(ref2) {
    var points = ref2.points;
    return points;
  })).forEach(function(ref2) {
    var x = ref2.x;
    var y = ref2.y;
    minX = Math.min(minX, x);
    minY = Math.min(minY, y);
    maxX = Math.max(maxX, x);
    maxY = Math.max(maxY, y);
  });
  return {
    minX,
    minY,
    width: maxX - minX,
    height: maxY - minY
  };
}
function s2b(points, prev, next) {
  var results = [];
  var bb1 = [0, 2 / 3, 1 / 3, 0];
  var bb2 = [0, 1 / 3, 2 / 3, 0];
  var bb3 = [0, 1 / 6, 2 / 3, 1 / 6];
  var dot4 = function(a2, b) {
    return a2[0] * b[0] + a2[1] * b[1] + a2[2] * b[2] + a2[3] * b[3];
  };
  var px = [points[points.length - 1].x, points[0].x, points[1].x, points[2].x];
  var py = [points[points.length - 1].y, points[0].y, points[1].y, points[2].y];
  results.push({
    type: prev === "M" ? "M" : "L",
    points: [{ x: dot4(bb3, px), y: dot4(bb3, py) }]
  });
  for (var i = 3; i < points.length; i++) {
    px = [points[i - 3].x, points[i - 2].x, points[i - 1].x, points[i].x];
    py = [points[i - 3].y, points[i - 2].y, points[i - 1].y, points[i].y];
    results.push({
      type: "C",
      points: [
        { x: dot4(bb1, px), y: dot4(bb1, py) },
        { x: dot4(bb2, px), y: dot4(bb2, py) },
        { x: dot4(bb3, px), y: dot4(bb3, py) }
      ]
    });
  }
  if (next === "L" || next === "C") {
    var last = points[points.length - 1];
    results.push({ type: "L", points: [{ x: last.x, y: last.y }] });
  }
  return results;
}
function toSVGPath(instructions) {
  return instructions.map(function(ref) {
    var type = ref.type;
    var points = ref.points;
    return type + points.map(function(ref2) {
      var x = ref2.x;
      var y = ref2.y;
      return x + "," + y;
    }).join(",");
  }).join("");
}
function compileDrawing(rawCommands) {
  var ref$1;
  var commands = [];
  var i = 0;
  while (i < rawCommands.length) {
    var arr = rawCommands[i];
    var cmd = createCommand(arr);
    if (isValid(cmd)) {
      if (cmd.type === "S") {
        var ref = (commands[i - 1] || { points: [{ x: 0, y: 0 }] }).points.slice(-1)[0];
        var x = ref.x;
        var y = ref.y;
        cmd.points.unshift({ x, y });
      }
      if (i) {
        cmd.prev = commands[i - 1].type;
        commands[i - 1].next = cmd.type;
      }
      commands.push(cmd);
      i++;
    } else {
      if (i && commands[i - 1].type === "S") {
        var additionPoints = {
          p: cmd.points,
          c: commands[i - 1].points.slice(0, 3)
        };
        commands[i - 1].points = commands[i - 1].points.concat(
          (additionPoints[arr[0]] || []).map(function(ref2) {
            var x2 = ref2.x;
            var y2 = ref2.y;
            return { x: x2, y: y2 };
          })
        );
      }
      rawCommands.splice(i, 1);
    }
  }
  var instructions = (ref$1 = []).concat.apply(
    ref$1,
    commands.map(function(ref2) {
      var type = ref2.type;
      var points = ref2.points;
      var prev = ref2.prev;
      var next = ref2.next;
      return type === "S" ? s2b(points, prev, next) : { type, points };
    })
  );
  return Object.assign({ instructions, d: toSVGPath(instructions) }, getViewBox(commands));
}
var tTags = [
  "fs",
  "fsp",
  "clip",
  "c1",
  "c2",
  "c3",
  "c4",
  "a1",
  "a2",
  "a3",
  "a4",
  "alpha",
  "fscx",
  "fscy",
  "fax",
  "fay",
  "frx",
  "fry",
  "frz",
  "fr",
  "be",
  "blur",
  "bord",
  "xbord",
  "ybord",
  "shad",
  "xshad",
  "yshad"
];
function compileTag(tag, key, presets) {
  var obj, obj$1, obj$2;
  if (presets === void 0) presets = {};
  var value = tag[key];
  if (value === void 0) {
    return null;
  }
  if (key === "pos" || key === "org") {
    return value.length === 2 ? (obj = {}, obj[key] = { x: value[0], y: value[1] }, obj) : null;
  }
  if (key === "move") {
    var x1 = value[0];
    var y1 = value[1];
    var x2 = value[2];
    var y2 = value[3];
    var t1 = value[4];
    if (t1 === void 0) t1 = 0;
    var t2 = value[5];
    if (t2 === void 0) t2 = 0;
    return value.length === 4 || value.length === 6 ? { move: { x1, y1, x2, y2, t1, t2 } } : null;
  }
  if (key === "fad" || key === "fade") {
    if (value.length === 2) {
      var t1$1 = value[0];
      var t2$1 = value[1];
      return { fade: { type: "fad", t1: t1$1, t2: t2$1 } };
    }
    if (value.length === 7) {
      var a1 = value[0];
      var a2 = value[1];
      var a3 = value[2];
      var t1$2 = value[3];
      var t2$2 = value[4];
      var t3 = value[5];
      var t4 = value[6];
      return { fade: { type: "fade", a1, a2, a3, t1: t1$2, t2: t2$2, t3, t4 } };
    }
    return null;
  }
  if (key === "clip") {
    var inverse = value.inverse;
    var scale = value.scale;
    var drawing = value.drawing;
    var dots = value.dots;
    if (drawing) {
      return { clip: { inverse, scale, drawing: compileDrawing(drawing), dots } };
    }
    if (dots) {
      var x1$1 = dots[0];
      var y1$1 = dots[1];
      var x2$1 = dots[2];
      var y2$1 = dots[3];
      return { clip: { inverse, scale, drawing, dots: { x1: x1$1, y1: y1$1, x2: x2$1, y2: y2$1 } } };
    }
    return null;
  }
  if (/^[xy]?(bord|shad)$/.test(key)) {
    value = Math.max(value, 0);
  }
  if (key === "bord") {
    return { xbord: value, ybord: value };
  }
  if (key === "shad") {
    return { xshad: value, yshad: value };
  }
  if (/^c\d$/.test(key)) {
    return obj$1 = {}, obj$1[key] = value || presets[key], obj$1;
  }
  if (key === "alpha") {
    return { a1: value, a2: value, a3: value, a4: value };
  }
  if (key === "fr") {
    return { frz: value };
  }
  if (key === "fs") {
    return {
      fs: /^\+|-/.test(value) ? (value * 1 > -10 ? 1 + value / 10 : 1) * presets.fs : value * 1
    };
  }
  if (key === "K") {
    return { kf: value };
  }
  if (key === "t") {
    var t1$3 = value.t1;
    var accel = value.accel;
    var tags = value.tags;
    var t2$3 = value.t2 || (presets.end - presets.start) * 1e3;
    var compiledTag = {};
    tags.forEach(function(t) {
      var k = Object.keys(t)[0];
      if (~tTags.indexOf(k) && !(k === "clip" && !t[k].dots)) {
        Object.assign(compiledTag, compileTag(t, k, presets));
      }
    });
    return { t: { t1: t1$3, t2: t2$3, accel, tag: compiledTag } };
  }
  return obj$2 = {}, obj$2[key] = value, obj$2;
}
var a2an = [
  null,
  1,
  2,
  3,
  null,
  7,
  8,
  9,
  null,
  4,
  5,
  6
];
var globalTags = ["r", "a", "an", "pos", "org", "move", "fade", "fad", "clip"];
function inheritTag(pTag) {
  return JSON.parse(JSON.stringify(Object.assign({}, pTag, {
    k: void 0,
    kf: void 0,
    ko: void 0,
    kt: void 0
  })));
}
function compileText(ref) {
  var styles = ref.styles;
  var style = ref.style;
  var parsed = ref.parsed;
  var start = ref.start;
  var end = ref.end;
  var alignment;
  var q = { q: styles[style].tag.q };
  var pos;
  var org;
  var move;
  var fade;
  var clip;
  var slices = [];
  var slice = { style, fragments: [] };
  var prevTag = {};
  for (var i = 0; i < parsed.length; i++) {
    var ref$1 = parsed[i];
    var tags = ref$1.tags;
    var text = ref$1.text;
    var drawing = ref$1.drawing;
    var reset = void 0;
    for (var j = 0; j < tags.length; j++) {
      var tag = tags[j];
      reset = tag.r === void 0 ? reset : tag.r;
    }
    var fragment = {
      tag: reset === void 0 ? inheritTag(prevTag) : {},
      text,
      drawing: drawing.length ? compileDrawing(drawing) : null
    };
    for (var j$1 = 0; j$1 < tags.length; j$1++) {
      var tag$1 = tags[j$1];
      alignment = alignment || a2an[tag$1.a || 0] || tag$1.an;
      q = compileTag(tag$1, "q") || q;
      pos = pos || compileTag(tag$1, "pos");
      org = org || compileTag(tag$1, "org");
      move = move || compileTag(tag$1, "move");
      fade = fade || compileTag(tag$1, "fade") || compileTag(tag$1, "fad");
      clip = compileTag(tag$1, "clip") || clip;
      var key = Object.keys(tag$1)[0];
      if (key && !~globalTags.indexOf(key)) {
        var sliceTag = styles[style].tag;
        var c1 = sliceTag.c1;
        var c2 = sliceTag.c2;
        var c3 = sliceTag.c3;
        var c4 = sliceTag.c4;
        var fs = prevTag.fs || sliceTag.fs;
        var compiledTag = compileTag(tag$1, key, { start, end, c1, c2, c3, c4, fs });
        if (key === "t") {
          fragment.tag.t = fragment.tag.t || [];
          fragment.tag.t.push(compiledTag.t);
        } else {
          Object.assign(fragment.tag, compiledTag);
        }
      }
    }
    prevTag = fragment.tag;
    if (reset !== void 0) {
      slices.push(slice);
      slice = { style: styles[reset] ? reset : style, fragments: [] };
    }
    if (fragment.text || fragment.drawing) {
      var prev = slice.fragments[slice.fragments.length - 1] || {};
      if (prev.text && fragment.text && !Object.keys(fragment.tag).length) {
        prev.text += fragment.text;
      } else {
        slice.fragments.push(fragment);
      }
    }
  }
  slices.push(slice);
  return Object.assign({ alignment, slices }, q, pos, org, move, fade, clip);
}
function compileDialogues(ref) {
  var styles = ref.styles;
  var dialogues = ref.dialogues;
  var minLayer = Infinity;
  var results = [];
  for (var i = 0; i < dialogues.length; i++) {
    var dia = dialogues[i];
    if (dia.Start >= dia.End) {
      continue;
    }
    if (!styles[dia.Style]) {
      dia.Style = "Default";
    }
    var stl = styles[dia.Style].style;
    var compiledText = compileText({
      styles,
      style: dia.Style,
      parsed: dia.Text.parsed,
      start: dia.Start,
      end: dia.End
    });
    var alignment = compiledText.alignment || stl.Alignment;
    minLayer = Math.min(minLayer, dia.Layer);
    results.push(Object.assign({
      layer: dia.Layer,
      start: dia.Start,
      end: dia.End,
      style: dia.Style,
      name: dia.Name,
      // reset style by `\r` will not effect margin and alignment
      margin: {
        left: dia.MarginL || stl.MarginL,
        right: dia.MarginR || stl.MarginR,
        vertical: dia.MarginV || stl.MarginV
      },
      effect: dia.Effect
    }, compiledText, { alignment }));
  }
  for (var i$1 = 0; i$1 < results.length; i$1++) {
    results[i$1].layer -= minLayer;
  }
  return results.sort(function(a2, b) {
    return a2.start - b.start || a2.end - b.end;
  });
}
var DEFAULT_STYLE = {
  Name: "Default",
  Fontname: "Arial",
  Fontsize: "20",
  PrimaryColour: "&H00FFFFFF&",
  SecondaryColour: "&H000000FF&",
  OutlineColour: "&H00000000&",
  BackColour: "&H00000000&",
  Bold: "0",
  Italic: "0",
  Underline: "0",
  StrikeOut: "0",
  ScaleX: "100",
  ScaleY: "100",
  Spacing: "0",
  Angle: "0",
  BorderStyle: "1",
  Outline: "2",
  Shadow: "2",
  Alignment: "2",
  MarginL: "10",
  MarginR: "10",
  MarginV: "10",
  Encoding: "1"
};
function parseStyleColor(color) {
  if (/^(&|H|&H)[0-9a-f]{6,}/i.test(color)) {
    var ref = color.match(/&?H?([0-9a-f]{2})?([0-9a-f]{6})/i);
    var a2 = ref[1];
    var c2 = ref[2];
    return [a2 || "00", c2];
  }
  var num = parseInt(color, 10);
  if (!isNaN(num)) {
    var min = -2147483648;
    var max = 2147483647;
    if (num < min) {
      return ["00", "000000"];
    }
    var aabbggrr = min <= num && num <= max ? ("00000000" + (num < 0 ? num + 4294967296 : num).toString(16)).slice(-8) : String(num).slice(0, 8);
    return [aabbggrr.slice(0, 2), aabbggrr.slice(2)];
  }
  return ["00", "000000"];
}
function compileStyles(ref) {
  var info = ref.info;
  var style = ref.style;
  var defaultStyle = ref.defaultStyle;
  var result = {};
  var styles = [Object.assign({}, defaultStyle, { Name: "Default" })].concat(style);
  var loop = function(i2) {
    var s = Object.assign({}, DEFAULT_STYLE, styles[i2]);
    if (/^(\*+)Default$/.test(s.Name)) {
      s.Name = "Default";
    }
    Object.keys(s).forEach(function(key) {
      if (key !== "Name" && key !== "Fontname" && !/Colour/.test(key)) {
        s[key] *= 1;
      }
    });
    var ref$1 = parseStyleColor(s.PrimaryColour);
    var a1 = ref$1[0];
    var c1 = ref$1[1];
    var ref$2 = parseStyleColor(s.SecondaryColour);
    var a2 = ref$2[0];
    var c2 = ref$2[1];
    var ref$3 = parseStyleColor(s.OutlineColour);
    var a3 = ref$3[0];
    var c3 = ref$3[1];
    var ref$4 = parseStyleColor(s.BackColour);
    var a4 = ref$4[0];
    var c4 = ref$4[1];
    var tag = {
      fn: s.Fontname,
      fs: s.Fontsize,
      c1,
      a1,
      c2,
      a2,
      c3,
      a3,
      c4,
      a4,
      b: Math.abs(s.Bold),
      i: Math.abs(s.Italic),
      u: Math.abs(s.Underline),
      s: Math.abs(s.StrikeOut),
      fscx: s.ScaleX,
      fscy: s.ScaleY,
      fsp: s.Spacing,
      frz: s.Angle,
      xbord: s.Outline,
      ybord: s.Outline,
      xshad: s.Shadow,
      yshad: s.Shadow,
      fe: s.Encoding,
      // TODO: [breaking change] remove `q` from style
      q: /^[0-3]$/.test(info.WrapStyle) ? info.WrapStyle * 1 : 2
    };
    result[s.Name] = { style: s, tag };
  };
  for (var i = 0; i < styles.length; i++) loop(i);
  return result;
}
function compile(text, options) {
  if (options === void 0) options = {};
  var tree = parse(text);
  var styles = compileStyles({
    info: tree.info,
    style: tree.styles.style,
    defaultStyle: options.defaultStyle || {}
  });
  return {
    info: tree.info,
    width: tree.info.PlayResX * 1 || null,
    height: tree.info.PlayResY * 1 || null,
    wrapStyle: /^[0-3]$/.test(tree.info.WrapStyle) ? tree.info.WrapStyle * 1 : 2,
    collisions: tree.info.Collisions || "Normal",
    styles,
    dialogues: compileDialogues({
      styles,
      dialogues: tree.events.dialogue
    })
  };
}
var drawingInstructionMap = {
  M: "m",
  L: "l",
  C: "b"
};
function decompileDrawing(ref) {
  var instructions = ref.instructions;
  return instructions.map(function(ref2) {
    var ref$1;
    var type = ref2.type;
    var points = ref2.points;
    return (ref$1 = [drawingInstructionMap[type]]).concat.apply(ref$1, points.map(function(ref3) {
      var x = ref3.x;
      var y = ref3.y;
      return [x, y];
    })).join(" ");
  }).join(" ");
}
var ca = function(x) {
  return function(n) {
    return function(_) {
      return "" + n + x + "&H" + _ + "&";
    };
  };
};
var c = ca("c");
var a = ca("a");
var tagDecompiler = {
  c1: c(1),
  c2: c(2),
  c3: c(3),
  c4: c(4),
  a1: a(1),
  a2: a(2),
  a3: a(3),
  a4: a(4),
  pos: function(_) {
    return "pos(" + [_.x, _.y] + ")";
  },
  org: function(_) {
    return "org(" + [_.x, _.y] + ")";
  },
  move: function(_) {
    return "move(" + [_.x1, _.y1, _.x2, _.y2, _.t1, _.t2] + ")";
  },
  fade: function(_) {
    return _.type === "fad" ? "fad(" + [_.t1, _.t2] + ")" : "fade(" + [_.a1, _.a2, _.a3, _.t1, _.t2, _.t3, _.t4] + ")";
  },
  clip: function(_) {
    return (_.inverse ? "i" : "") + "clip(" + (_.dots ? "" + [_.dots.x1, _.dots.y1, _.dots.x2, _.dots.y2] : (_.scale === 1 ? "" : _.scale + ",") + decompileDrawing(_.drawing)) + ")";
  },
  // eslint-disable-next-line no-use-before-define
  t: function(arr) {
    return arr.map(function(_) {
      return "t(" + [_.t1, _.t2, _.accel, decompileTag(_.tag)] + ")";
    }).join("\\");
  }
};
function decompileTag(tag) {
  return Object.keys(tag).map(function(key) {
    var fn = tagDecompiler[key] || function(_) {
      return "" + key + _;
    };
    return "\\" + fn(tag[key]);
  }).join("");
}

// src/utils.ts
function convertAegisubColorToHex(aegisubColor) {
  const colorValue = aegisubColor.replace(/&H|&/g, "");
  const blue = parseInt(colorValue.slice(2, 4), 16);
  const green = parseInt(colorValue.slice(4, 6), 16);
  const red = parseInt(colorValue.slice(6, 8), 16);
  const rgba = `rgb(${red}, ${green}, ${blue}, 1)`;
  const hex = rgbaToHex(rgba);
  return hex;
}
function swapBBGGRR(color) {
  const colorValue = color.replace("#", "");
  const blue = colorValue.slice(0, 2);
  const green = colorValue.slice(2, 4);
  const red = colorValue.slice(4, 6);
  return `#${red}${green}${blue}`;
}
function rgbaToHex(rgba) {
  const components = rgba.match(/\d+/g);
  let red = parseInt(components[0]);
  let green = parseInt(components[1]);
  let blue = parseInt(components[2]);
  let alpha = parseFloat(components[3]);
  let redHex = red.toString(16).padStart(2, "0");
  let greenHex = green.toString(16).padStart(2, "0");
  let blueHex = blue.toString(16).padStart(2, "0");
  let rgbHex = "#" + redHex + greenHex + blueHex;
  if (alpha !== 1) {
    let alphaHex = Math.round(alpha * 255).toString(16).padStart(2, "0");
    return rgbHex + alphaHex;
  }
  return rgbHex;
}
function ruleOfThree(value, valueMin) {
  return valueMin * 100 / value;
}
function genRandomString(ln) {
  const characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
  let randomString = "";
  for (let i = 0; i < ln; i++) {
    const randomIndex = Math.floor(Math.random() * characters.length);
    randomString += characters.charAt(randomIndex);
  }
  return randomString;
}
function randomId(parts, separator = "-", prefix = "", ln = 10) {
  let partsArray = [];
  for (let i = 0; i < parts; i++) {
    partsArray.push(genRandomString(ln));
  }
  return prefix + partsArray.join(separator);
}
function stringHash(str) {
  let hash = 0;
  if (str.length == 0) return hash;
  for (let i = 0; i < str.length; i++) {
    let char = str.charCodeAt(i);
    hash = (hash << 5) - hash + char;
    hash = hash & hash;
  }
  return hash;
}
function chunkCharWidth(chars) {
  let w = 0;
  for (let i = 0; i < chars.length; i++) {
    let char = chars[i];
    if (char.kind == 1 /* NORMAL */) {
      w += char.w;
    }
  }
  return w;
}
function chunkCharToString(chars) {
  let w = "";
  for (let i = 0; i < chars.length; i++) {
    let char = chars[i];
    if (char.kind == 1 /* NORMAL */) {
      w += char.c;
    }
  }
  return w;
}
function newRender(top, left, width, height, zIndex, insertAfter) {
  const render = document.createElement("div");
  render.id = randomId(2, "-", "ASSRendererRender-", 5);
  render.style.position = "absolute";
  render.style.width = width + "px";
  render.style.height = height + "px";
  render.style.top = top + "px";
  render.style.left = left + "px";
  render.style.pointerEvents = "none";
  render.style.overflow = "hidden";
  render.style.boxSizing = "border-box";
  render.style.padding = "0px";
  render.style.margin = "0px";
  if (zIndex) {
    render.style.zIndex = zIndex.toString();
  }
  if (insertAfter) {
    insertAfter.after(render);
  } else {
    document.body.appendChild(render);
  }
  return render;
}
function newCanvas(width, height, dataLayer = 0, layerName, appendIn, insertAfter) {
  const canvas = document.createElement("canvas");
  canvas.id = randomId(2, "-", "Canvas-", 5);
  canvas.style.position = "absolute";
  canvas.style.top = "0px";
  canvas.style.left = "0px";
  canvas.style.pointerEvents = "none";
  canvas.width = width;
  canvas.height = height;
  canvas.dataset.layer = dataLayer.toString();
  canvas.dataset.identifier = uuidgen();
  if (layerName) {
    canvas.dataset.name = layerName;
  }
  if (insertAfter) {
    insertAfter.after(canvas);
  } else {
    if (!appendIn) {
      appendIn = document.body;
    }
    appendIn.appendChild(canvas);
  }
  return canvas;
}
function uuidgen() {
  const v4 = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx";
  return v4.replace(/[xy]/g, (c2) => {
    const r = Math.random() * 16 | 0;
    const v = c2 === "x" ? r : r & 3 | 8;
    return v.toString(16);
  });
}
function parseAlpha(of) {
  const a2 = `0x${of}`;
  return parseInt(a2, 16);
}
function blendAlpha(color, alpha) {
  color = color.replace("#", "");
  const red = parseInt(color.substring(0, 2), 16);
  const green = parseInt(color.substring(2, 4), 16);
  const blue = parseInt(color.substring(4, 6), 16);
  return `rgba(${red}, ${green}, ${blue}, ${alpha == 0 ? 1 : alpha / 255})`;
}
var Vector2 = class _Vector2 {
  x;
  y;
  constructor(x = 0, y = 0) {
    this.x = x;
    this.y = y;
  }
  add(rhs) {
    return new _Vector2(this.x + rhs.x, this.y + rhs.y);
  }
  sub(rhs) {
    return new _Vector2(this.x - rhs.x, this.y - rhs.y);
  }
  mul(i) {
    return new _Vector2(this.x * i, this.y * i);
  }
};
function vectorLerp(start, end, t) {
  return start.add(end.sub(start).mul(t));
}
function lerp(start, end, t) {
  return start + (end - start) * t;
}
function getOpacity(fade, startTime, endTime, time) {
  const fadeIn = startTime + fade.fadein;
  const fadeOut = endTime - fade.fadeout;
  if (time < fadeIn) {
    let t = (time - startTime) / (fadeIn - startTime);
    return lerp(0, 1, t);
  } else if (time >= fadeIn && time < fadeOut) {
    return 1;
  } else if (time >= fadeOut && time <= endTime) {
    let t = (time - fadeOut) / (endTime - fadeOut);
    return lerp(1, 0, t);
  } else {
    return -1;
  }
}
function getOpacityComplex(fade, startTime, endTime, time) {
  console.debug(fade);
  const t1 = startTime + fade.t1;
  const t2 = startTime + fade.t2;
  const t3 = startTime + fade.t3;
  const t4 = startTime + fade.t4;
  if (time < t1) {
    return fade.a1;
  } else if (time >= t1 && time < t2) {
    let t = (time - t1) / (t2 - t1);
    return lerp(fade.a1, fade.a2, t);
  } else if (time >= t2 && time < t3) {
    return fade.a2;
  } else if (time >= t3 && time < t4) {
    let t = (time - t3) / (t4 - t3);
    return lerp(fade.a2, fade.a3, t);
  } else if (time >= t4 && time <= endTime) {
    return fade.a3;
  } else {
    return -1;
  }
}

// src/renderer.ts
var Renderer = class {
  compiledASS;
  renderDiv;
  layers;
  numberOfLayers;
  video;
  playerResX;
  playerResY;
  stop = false;
  animationHandle = null;
  _log;
  ppass = [];
  styles;
  collisions = "Normal";
  dt = 0;
  previous = 0;
  currentTime = 0;
  constructor(ass, sizes, video, log, zIndex) {
    this._log = log;
    this.compiledASS = ass;
    this.styles = ass.styles;
    this.collisions = ass.collisions;
    if (this._log === "DEBUG") {
      this.log("DEBUG", this.compiledASS);
    }
    this.playerResX = this.compiledASS.width;
    this.playerResY = this.compiledASS.height;
    this.renderDiv = newRender(sizes.y, sizes.x, sizes.width, sizes.height, zIndex, video);
    const background = newCanvas(sizes.width, sizes.height, -1, "background", this.renderDiv);
    const bgCtx = background.getContext("2d");
    if (bgCtx === null) {
      throw new Error("Unable to initilize the Canvas 2D context");
    }
    this.layers = [
      {
        canvas: background,
        ctx: bgCtx
      }
    ];
    this.numberOfLayers = this.findTotalLayers(ass);
    this.insertLayers(sizes, background);
    this.video = video;
  }
  insertLayers(sizes, insertAfter) {
    for (let i = 0; i < this.numberOfLayers; i++) {
      const canvas = newCanvas(sizes.width, sizes.height, i, "frame", void 0, insertAfter);
      const ctx = canvas.getContext("2d");
      if (ctx === null) {
        throw new Error("Unable to initilize the Canvas 2D context");
      }
      ctx.textRendering = "geometricPrecision";
      this.layers.push({
        canvas,
        ctx
      });
    }
  }
  getLayer(l) {
    for (let i = 1; i < this.layers.length; i++) {
      if (this.layers[i]?.canvas.dataset.layer == l.toString()) {
        return this.layers[i];
      }
    }
    return null;
  }
  findTotalLayers(ass) {
    let maxLayer = 1;
    ass.dialogues.forEach((dialogue) => {
      if (dialogue.layer >= maxLayer) maxLayer++;
    });
    return maxLayer;
  }
  async warmup() {
    const { dialogues } = this.compiledASS;
    const canvas = document.createElement("canvas");
    canvas.height = this.playerResY;
    canvas.width = this.playerResX;
    const ctx = canvas.getContext("2d");
    const computelayer = {
      ctx,
      canvas
    };
    computelayer.canvas.width = this.playerResX;
    computelayer.canvas.height = this.playerResY;
    console.debug(dialogues);
    for (let i = 0; i < dialogues.length; i++) {
      const dia = dialogues[i];
      const { layer, alignment, start, end, style, margin, slices, pos, move, org, fade } = dia;
      let movAnim;
      let rotOrg;
      let fadeAnim;
      if (move) {
        movAnim = {
          from: new Vector2(move.x1, move.y1),
          to: new Vector2(move.x2, move.y2),
          start: move.t1,
          end: move.t2
        };
      }
      if (org) {
        rotOrg = new Vector2(org.x, org.y);
      }
      if (fade) {
        switch (fade.type) {
          case "fad":
            fadeAnim = { type: 0 /* Simple */, fadein: fade.t1, fadeout: fade.t2 };
            break;
          case "fade":
            let { a1, a2, a3, t1, t2, t3, t4 } = fade;
            fadeAnim = { type: 1 /* Complex */, a1, a2, a3, t1, t2, t3, t4 };
            break;
        }
      }
      this.ppass.push({
        start,
        layer,
        end,
        style,
        margin,
        alignment,
        pos,
        move: movAnim,
        fade: fadeAnim,
        rotationOrigin: rotOrg,
        chars: this.processSlices(alignment, slices, computelayer)
      });
    }
  }
  processSlices(alignment, slices, layer) {
    let chars = [];
    slices.forEach((slice) => {
      const { style, fragments } = slice;
      let font = this.computeStyle(style, alignment, layer);
      let _frag = [];
      fragments.forEach((frag) => {
        this.applyOverrideTag(frag.tag, font);
        this.applyFont(font, layer);
        const text = frag.text.replaceAll("\\N", "\n").split("");
        let ord = 0;
        let chars2 = [];
        for (let i = 0; i < text.length; i++) {
          const char = text[i];
          if (char == "\n") {
            chars2.push({
              kind: 0 /* NEWLINE */
            });
            ord = 0;
          } else {
            chars2.push({
              kind: 1 /* NORMAL */,
              pos: new Vector2(),
              c: char,
              ord,
              w: 0,
              style,
              tag: frag.tag
            });
            ord += 1;
          }
        }
        _frag = _frag.concat(chars2);
      });
      chars = chars.concat(_frag);
    });
    return chars;
  }
  sublog(type, ...message) {
    switch (type) {
      case "DISABLE":
        break;
      case "VERBOSE":
        console.info(type, ...message);
        break;
      case "DEBUG":
        console.debug(type, ...message);
        break;
      case "WARN":
        console.warn(type, ...message);
        break;
    }
  }
  log(type, ...message) {
    switch (this._log) {
      case "DISABLE":
        break;
      case "VERBOSE":
        this.sublog(type, message);
        break;
      case "DEBUG":
        if (type == "DEBUG") {
          this.sublog(type, message);
        }
        break;
      case "WARN":
        if (type == "WARN") {
          this.sublog(type, message);
        }
        break;
    }
  }
  async startRendering() {
    window.requestAnimationFrame((timestamp) => {
      this.previous = timestamp;
      this.animationHandle = requestAnimationFrame(this.render.bind(this));
    });
  }
  render(timestamp) {
    if (this.stop === true) {
      if (this.animationHandle) {
        this.renderDiv.remove();
        cancelAnimationFrame(this.animationHandle);
      }
    } else {
      this.dt = (timestamp - this.previous) / 1e3;
      this.previous = timestamp;
      this.display(this.video.currentTime);
      this.animationHandle = requestAnimationFrame(this.render.bind(this));
    }
  }
  destroy() {
    this.stop = true;
  }
  display(time) {
    this.currentTime = time;
    this.clear();
    const slicesToDisplay = this.getSlices(time);
    slicesToDisplay.forEach((dialogue) => {
      this.showDialogue(dialogue, time);
    });
  }
  charsToString(chars) {
    return chars.reduce((acc, v) => {
      return acc += v.kind == 0 /* NEWLINE */ ? "\n" : v.c;
    }, "");
  }
  getSlices(time) {
    return this.ppass.filter((dialogue) => {
      return dialogue.start <= time && dialogue.end >= time;
    });
  }
  getLineHeights(layer, lines) {
    let heights = [];
    let currentMaxHeight = 0;
    lines.forEach((line) => {
      line.forEach((char) => {
        if (char.kind === 1 /* NORMAL */) {
          let font = this.computeStyle(char.style, 1, layer);
          this.applyOverrideTag(char.tag, font);
          this.applyFont(font, layer);
          let calc = layer.ctx.measureText(char.c).fontBoundingBoxAscent + layer.ctx.measureText(char.c).fontBoundingBoxDescent;
          if (calc > currentMaxHeight) {
            currentMaxHeight = calc;
          }
        }
      });
      heights.push(currentMaxHeight);
      currentMaxHeight = 0;
    });
    return heights;
  }
  countLines(chars) {
    return chars.filter((c2) => c2.kind === 0 /* NEWLINE */).length + 1;
  }
  lineWidth(layer, chars) {
    let w = 0;
    chars.forEach((char) => {
      if (char.kind === 1 /* NORMAL */) {
        let font = this.computeStyle(char.style, 1, layer);
        this.applyOverrideTag(char.tag, font);
        this.applyFont(font, layer);
        w += layer.ctx.measureText(char.c).width + font.t.fsp;
      }
    });
    return w;
  }
  lines(chars) {
    let lines = [];
    let buff = [];
    chars.forEach((char) => {
      switch (char.kind) {
        case 0 /* NEWLINE */:
          lines.push(buff);
          buff = [];
          break;
        case 1 /* NORMAL */:
          buff.push(char);
          break;
      }
    });
    if (buff.length > 0) {
      lines.push(buff);
    }
    return lines;
  }
  showDialogue(d, time) {
    const layer = this.getLayer(d.layer);
    if (layer) {
      let font = this.computeStyle(d.style, d.alignment, layer);
      const lines = this.lines(d.chars);
      const lineHeights = this.getLineHeights(layer, lines);
      const lineHeight = Math.max(...lineHeights);
      const linesCount = this.countLines(d.chars);
      const totalHeight = lineHeight * linesCount;
      let cX = 0;
      let cY = 0;
      let baseX = 0;
      let currentOpacity = 1;
      let customPosition = false;
      let rotationOrigin;
      const Xratio = layer.canvas.width / this.playerResX;
      const Yratio = layer.canvas.height / this.playerResY;
      if (d.pos) {
        customPosition = true;
        baseX = d.pos.x * Xratio;
        cX = baseX;
        cY = d.pos.y * Yratio;
      }
      if (d.move) {
        customPosition = true;
        let startPosition = new Vector2(d.move.from.x * Xratio, d.move.from.y * Yratio);
        let endPosition = new Vector2(d.move.to.x * Xratio, d.move.to.y * Yratio);
        let actualPosition = vectorLerp(
          startPosition,
          endPosition,
          (time - d.start) / (d.end - d.start)
        );
        baseX = actualPosition.x;
        cX = baseX;
        cY = actualPosition.y;
      }
      if (d.rotationOrigin)
        rotationOrigin = new Vector2(d.rotationOrigin.x * Xratio, d.rotationOrigin.y * Yratio);
      if (d.fade) {
        switch (d.fade.type) {
          case 0 /* Simple */:
            currentOpacity = getOpacity(d.fade, d.start * 1e3, d.end * 1e3, time * 1e3);
            break;
          case 1 /* Complex */:
            currentOpacity = getOpacityComplex(d.fade, d.start * 1e3, d.end * 1e3, time * 1e3);
            break;
        }
      }
      d.chars.forEach((char) => {
        switch (char.kind) {
          case 0 /* NEWLINE */:
            cX = baseX;
            cY += lineHeight;
            break;
          case 1 /* NORMAL */:
            font = this.computeStyle(char.style, d.alignment, layer);
            this.applyOverrideTag(char.tag, font);
            this.applyFont(font, layer);
            const w = layer.ctx.measureText(char.c).width + font.t.fsp;
            char.pos.x = cX;
            char.pos.y = cY;
            char.w = w;
            cX += w;
            break;
        }
      });
      const margin = this.upscaleMargin(d.margin);
      lines.forEach((line) => {
        const lineWidth = this.lineWidth(layer, line);
        if (!customPosition) {
          switch (font.textAlign) {
            case 0 /* Left */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */) char.pos.x += margin.left;
              });
              break;
            case 1 /* Center */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */)
                  char.pos.x += (layer.canvas.width - lineWidth) / 2;
              });
              break;
            case 2 /* Right */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */)
                  char.pos.x += layer.canvas.width - lineWidth - margin.right;
              });
              break;
            default:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */) char.pos.x += margin.left;
              });
              break;
          }
          switch (font.textBaseline) {
            case 0 /* Bottom */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */)
                  char.pos.y += layer.canvas.height - (lines.length > 1 ? totalHeight / lines.length : 0) - margin.vertical;
              });
              break;
            case 1 /* Middle */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */)
                  char.pos.y += (layer.canvas.height - totalHeight) / 2 + (lines.length > 1 ? totalHeight / lines.length : lineHeight);
              });
              break;
            case 2 /* Top */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */) char.pos.y += margin.vertical + lineHeight / 2;
              });
              break;
            default:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */)
                  char.pos.y += layer.canvas.height - (lines.length > 1 ? totalHeight / lines.length : 0) - margin.vertical;
              });
              break;
          }
        } else {
          switch (font.textAlign) {
            case 0 /* Left */:
              break;
            case 1 /* Center */:
              line.forEach((char) => {
                if (char.kind == 1 /* NORMAL */) char.pos.x -= lineWidth / 2;
              });
              break;
            case 2 /* Right */:
              break;
            default:
              break;
          }
        }
      });
      let currentWord = [];
      let currentFont = null;
      let words = [];
      let currentHash = 0;
      d.chars.forEach((char) => {
        if (char.kind == 1 /* NORMAL */) {
          let font2 = this.computeStyle(char.style, d.alignment, layer);
          this.applyOverrideTag(char.tag, font2);
          let fHash = this.getFontHash(font2);
          if (currentHash !== fHash) {
            if (currentWord.length > 0) {
              if (currentFont !== null) {
                words.push({
                  font: currentFont,
                  value: currentWord,
                  w: chunkCharWidth(currentWord)
                });
                currentWord = [char];
                currentFont = font2;
                currentHash = fHash;
              }
            } else {
              currentFont = font2;
              currentWord.push(char);
              currentHash = fHash;
            }
          } else {
            currentFont = font2;
            currentWord.push(char);
            currentHash = fHash;
          }
        } else {
          if (currentFont !== null) {
            words.push({
              font: currentFont,
              value: currentWord,
              w: chunkCharWidth(currentWord)
            });
            currentWord = [];
            currentFont = null;
          }
        }
      });
      if (currentWord.length > 0) {
        if (currentFont !== null) {
          words.push({
            font: currentFont,
            value: currentWord,
            w: chunkCharWidth(currentWord)
          });
          currentWord = [];
          currentFont = null;
        }
      }
      words.forEach((word) => {
        word.font.opacity = currentOpacity;
        layer.ctx.save();
        this.drawWord(word, time, d.start, layer);
        layer.ctx.restore();
      });
    }
  }
  getFontHash(font) {
    return stringHash(JSON.stringify(font));
  }
  drawWord(word, time, startTime, layer, debug = false) {
    let str = chunkCharToString(word.value);
    for (let i = 0; i < word.font.customAnimations.length; i++) {
      const ca2 = word.font.customAnimations[i];
      if (startTime * 1e3 + ca2.t1 <= time * 1e3 && time * 1e3 <= startTime * 1e3 + ca2.t2) {
        const end = startTime * 1e3 + ca2.t2;
        const start = startTime * 1e3 + ca2.t1;
        const t = (time * 1e3 - start) / (end - start);
        for (const field in ca2.tag) {
          switch (field) {
            case "a1":
            case "a3":
            case "a4":
              word.font.colors[field] = lerp(
                255,
                Math.abs(parseAlpha(ca2.tag[field]) - 255),
                t
              );
              break;
            case "blur":
            case "xshad":
            case "yshad":
            case "xbord":
            case "ybord":
              word.font[field] = lerp(0, ca2.tag[field], t);
              break;
            case "fax":
            case "fay":
            case "frx":
            case "fry":
            case "frz":
            case "fscx":
            case "fscy":
              word.font.t[field] = lerp(0, ca2.tag[field], t);
              break;
            case "fs":
              word.font.fontsize = this.upscale(
                lerp(0, ca2.tag[field], t),
                this.playerResY,
                this.layers[0]?.canvas.height || 0
              );
              break;
            case "fsp":
              word.font.t.fsp = this.upscale(
                lerp(0, ca2.tag[field], t),
                this.playerResX,
                this.layers[0]?.canvas.width || this.playerResX
              );
              break;
            default:
              break;
          }
        }
      }
    }
    this.applyFont(word.font, layer);
    const wordHead = word.value[0];
    if (wordHead.kind === 1 /* NORMAL */) {
      if (word.font.borderStyle !== 3) {
        if (word.font.xbord !== 0 || word.font.ybord !== 0) {
          layer.ctx.strokeText(str, wordHead.pos.x, wordHead.pos.y);
        }
      }
      let metrics = layer.ctx.measureText(str);
      layer.ctx.fillText(str, wordHead.pos.x, wordHead.pos.y);
      if (word.font.underline) {
        const y = wordHead.pos.y + metrics.emHeightDescent;
        layer.ctx.fillRect(wordHead.pos.x, y, metrics.width, layer.canvas.height * 0.2 / 100);
      }
      if (word.font.strikeout) {
        const y = wordHead.pos.y - metrics.hangingBaseline / 2;
        layer.ctx.shadowOffsetX = 0;
        layer.ctx.shadowOffsetY = 0;
        layer.ctx.fillRect(wordHead.pos.x, y, metrics.width, layer.canvas.height * 0.2 / 100);
      }
      if (word.font.borderStyle === 3) {
        this.drawTextBackground(
          wordHead.pos,
          metrics.actualBoundingBoxAscent + metrics.fontBoundingBoxDescent,
          metrics.width,
          word.font
        );
      }
    }
  }
  drawTextBackground(pos, height, width, font) {
    const layer = this.layers[0];
    layer.ctx.save();
    layer.ctx.beginPath();
    layer.ctx.fillStyle = font.colors.c2;
    layer.ctx.fillRect(pos.x, pos.y - height, width, height);
    layer.ctx.closePath();
    layer.ctx.restore();
  }
  upscaleY(y, baseCanvasHeight) {
    const canvasHeight = this.layers[0]?.canvas.height || this.playerResY;
    return canvasHeight * y / baseCanvasHeight;
  }
  upscaleX(x, baseCanvasWidth) {
    const canvasWidth = this.layers[0]?.canvas.width || this.playerResX;
    return canvasWidth * x / baseCanvasWidth;
  }
  clearLayer(layer) {
    this.layers[layer]?.ctx.clearRect(
      0,
      0,
      this.layers[layer]?.canvas.width,
      this.layers[layer]?.canvas.height
    );
  }
  clear() {
    this.layers.forEach((layer) => {
      layer.ctx.clearRect(0, 0, layer.canvas.width, layer.canvas.height);
    });
  }
  upscalePosition(pos) {
    return {
      x: this.upscale(pos.x, this.playerResX, this.layers[0]?.canvas.width || this.playerResX),
      y: this.upscale(pos.y, this.playerResY, this.layers[0]?.canvas.height || this.playerResY)
    };
  }
  upscaleMargin(margin) {
    return {
      left: this.upscale(
        margin.left,
        this.playerResX,
        this.layers[0]?.canvas.width || this.playerResX
      ),
      right: this.upscale(
        margin.right,
        this.playerResX,
        this.layers[0]?.canvas.width || this.playerResX
      ),
      vertical: this.upscale(
        margin.vertical,
        this.playerResY,
        this.layers[0]?.canvas.height || this.playerResY
      )
    };
  }
  applyOverrideTag(tag, font) {
    if (tag.b !== void 0) font.bold = tag.b === 1;
    if (tag.i !== void 0) font.italic = tag.i === 1;
    if (tag.u !== void 0) font.underline = tag.u === 1;
    if (tag.s !== void 0) font.strikeout = tag.s === 1;
    if (tag.fn !== void 0) font.fontname = tag.fn;
    if (tag.q !== void 0) font.t.q = tag.q;
    if (tag.fs !== void 0)
      font.fontsize = this.upscale(tag.fs, this.playerResY, this.layers[0]?.canvas.height || 0);
    if (tag.fscx !== void 0) font.t.fscx = tag.fscx;
    if (tag.fscy !== void 0) font.t.fscy = tag.fscy;
    if (tag.frz !== void 0) font.t.frz = tag.frz;
    if (tag.frx !== void 0) font.t.frx = tag.frx;
    if (tag.fry !== void 0) font.t.fry = tag.fry;
    if (tag.fax !== void 0) font.t.fax = tag.fax;
    if (tag.fay !== void 0) font.t.fay = tag.fay;
    if (tag.fsp !== void 0) {
      font.t.fsp = this.upscale(
        tag.fsp,
        this.playerResX,
        this.layers[0]?.canvas.width || this.playerResX
      );
    }
    if (tag.t !== void 0) font.customAnimations = tag.t;
    if (tag.c1 !== void 0) font.colors.c1 = swapBBGGRR(tag.c1);
    if (tag.c3 !== void 0) font.colors.c3 = swapBBGGRR(tag.c3);
    if (tag.c4 !== void 0) font.colors.c4 = swapBBGGRR(tag.c4);
    if (tag.a1 !== void 0) font.colors.a1 = parseAlpha(tag.a1);
    if (tag.a3 !== void 0) font.colors.a3 = parseAlpha(tag.a3);
    if (tag.a4 !== void 0) font.colors.a4 = parseAlpha(tag.a4);
    if (tag.xshad !== void 0) font.xshad = tag.xshad;
    if (tag.yshad !== void 0) font.yshad = tag.yshad;
    if (tag.xbord !== void 0) font.xbord = tag.xbord;
    if (tag.ybord !== void 0) font.ybord = tag.ybord;
    if (tag.blur !== void 0) font.blur = tag.blur;
  }
  upscale(x, firstcomp, secondcomp) {
    return ruleOfThree(firstcomp, secondcomp) * x / 100;
  }
  fontDecriptorString(font) {
    return `${font.bold ? "bold " : ""}${font.italic ? "italic " : ""}${font.fontsize.toFixed(
      3
    )}px "${font.fontname}"`;
  }
  computeStyle(name, alignment, layer) {
    const style = this.styles[name];
    if (style === void 0) {
      this.log("WARN", `[ass-html5:renderer] no corresponding style "${name}" found`);
    }
    const {
      fn,
      // font name
      fs,
      // font size
      a1,
      // primary alpha
      a2,
      // secondary alpha
      a3,
      // outline alpha
      c4,
      // shadow color
      a4,
      // shadow alpha
      b,
      // bold
      i,
      // italic
      u,
      // underline
      s,
      // strikeout
      fscx,
      // font scale x
      fscy,
      // font scale y
      fsp,
      // font spacing
      frz,
      // font rotation z
      xbord,
      // x border
      ybord,
      // y border
      xshad,
      // x shadow
      yshad,
      // y shadow
      fe,
      // font encoding
      q
      // wrap style
    } = style.tag;
    const { PrimaryColour, OutlineColour, SecondaryColour, BorderStyle } = style.style;
    const font = {
      fontsize: this.upscale(fs, this.playerResY, this.layers[0]?.canvas.height || this.playerResY),
      fontname: fn,
      bold: b === 1,
      italic: i === 1,
      underline: u === 1,
      strikeout: s === 1,
      colors: {
        c1: convertAegisubColorToHex(PrimaryColour),
        c2: convertAegisubColorToHex(SecondaryColour),
        c3: convertAegisubColorToHex(OutlineColour),
        c4,
        a1: parseAlpha(a1),
        a2: parseAlpha(a2),
        a3: parseAlpha(a3),
        a4: parseAlpha(a4)
      },
      t: {
        fscx,
        fscy,
        frz,
        frx: 0,
        fry: 0,
        fsp: this.upscale(fsp, this.playerResX, layer.canvas.width),
        q
      },
      customAnimations: [],
      xbord,
      ybord,
      xshad,
      yshad,
      blur: 0,
      fe,
      borderStyle: BorderStyle,
      opacity: 1,
      textAlign: this.getAlignment(alignment),
      textBaseline: this.getBaseLine(alignment)
    };
    this.applyFont(font, layer);
    return font;
  }
  applyFont(font, layer) {
    layer.ctx.font = this.fontDecriptorString(font);
    layer.ctx.fillStyle = blendAlpha(font.colors.c1, font.colors.a1);
    layer.ctx.strokeStyle = blendAlpha(font.colors.c3, font.colors.a3);
    layer.ctx.letterSpacing = `${font.t.fsp}px`;
    layer.ctx.shadowOffsetX = this.upscale(
      font.xshad,
      this.playerResX,
      this.layers[0]?.canvas.width || this.playerResX
    );
    layer.ctx.shadowOffsetY = this.upscale(
      font.yshad,
      this.playerResY,
      this.layers[0]?.canvas.height || this.playerResY
    );
    layer.ctx.shadowBlur = this.upscale(
      font.blur,
      this.playerResY,
      this.layers[0]?.canvas.height || this.playerResY
    );
    layer.ctx.shadowColor = blendAlpha(font.colors.c4, font.colors.a4);
    layer.ctx.lineWidth = this.upscale(font.xbord, this.playerResX, this.layers[0]?.canvas.width || this.playerResX) + this.upscale(font.ybord, this.playerResY, this.layers[0]?.canvas.height || this.playerResY);
    layer.ctx.lineCap = "round";
    layer.ctx.lineJoin = "round";
    layer.ctx.globalAlpha = font.opacity;
  }
  getAlignment(alignment) {
    switch (alignment) {
      case 1:
      case 4:
      case 7:
        return 0 /* Left */;
      case 2:
      case 5:
      case 8:
        return 1 /* Center */;
      case 3:
      case 6:
      case 9:
        return 2 /* Right */;
      default:
        return 3 /* Start */;
    }
  }
  getBaseLine(alignment) {
    switch (alignment) {
      case 1:
      case 2:
      case 3:
        return 0 /* Bottom */;
      case 4:
      case 5:
      case 6:
        return 1 /* Middle */;
      case 7:
      case 8:
      case 9:
        return 2 /* Top */;
      default:
        return 3 /* Alphabetic */;
    }
  }
};

// src/ass.ts
var ASS = class {
  assText;
  video;
  videoElement = null;
  renderer = null;
  ro = null;
  fonts;
  zIndex;
  onReady;
  logging = "DISABLE";
  compiledAss;
  constructor(options) {
    this.assText = options.assText;
    this.compiledAss = compile(this.assText, {});
    this.video = options.video;
    this.fonts = options.fonts;
    this.zIndex = options.zIndex;
    this.onReady = options.onReady;
    if (options.logging) this.logging = options.logging;
  }
  /**
   * Start the ass rendering process
   */
  async render() {
    if (typeof this.video == "string") {
      this.videoElement = document.querySelector(this.video);
      if (this.videoElement === null) {
        throw new Error("Unable to find the video element");
      }
    } else {
      this.videoElement = this.video;
    }
    const sizes = this.setCanvasSize();
    if (typeof this.fonts !== "undefined") {
      await this.loadFonts(this.fonts);
    }
    if (this.videoElement) {
      this.renderer = new Renderer(
        this.compiledAss,
        sizes,
        this.videoElement,
        this.logging,
        this.zIndex
      );
      this.setCanvasSize();
      this.videoElement.addEventListener("loadedmetadata", this.setCanvasSize.bind(this));
      this.ro = new ResizeObserver(this.setCanvasSize.bind(this));
      this.ro.observe(this.videoElement);
      await this.renderer.warmup();
      if (this.onReady) {
        this.onReady();
      }
      await this.renderer.startRendering();
    }
  }
  /**
   * Stop the rendering
   */
  destroy() {
    this.videoElement?.removeEventListener("loadedmetadata", this.setCanvasSize.bind(this));
    this.ro?.disconnect();
    this.renderer?.destroy();
    this.renderer = null;
  }
  setCanvasSize() {
    const { videoWidth, videoHeight, offsetTop, offsetLeft } = this.videoElement;
    const aspectRatio = videoWidth / videoHeight;
    const maxWidth = this.videoElement?.clientWidth || 0;
    const maxHeight = this.videoElement?.clientHeight || 0;
    let width = maxWidth;
    let height = maxHeight;
    let x = offsetLeft;
    let y = offsetTop;
    if (maxHeight * aspectRatio > maxWidth) {
      width = maxWidth;
      height = width / aspectRatio;
      y += (maxHeight - height) / 2;
    } else {
      height = maxHeight;
      width = height * aspectRatio;
      x += (maxWidth - width) / 2;
    }
    const sizes = {
      width,
      height,
      x,
      y
    };
    if (this.renderer?.renderDiv) {
      this.renderer.renderDiv.style.width = width + "px";
      this.renderer.renderDiv.style.height = height + "px";
      this.renderer.renderDiv.style.top = y + "px";
      this.renderer.renderDiv.style.left = x + "px";
    }
    this.renderer?.layers.forEach((layer) => {
      layer.canvas.width = width;
      layer.canvas.height = height;
    });
    return sizes;
  }
  async loadFonts(fonts) {
    for (const font of fonts) {
      try {
        const loaded = await this.loadFont(font);
        if (loaded) {
          if (this.logging == "VERBOSE")
            console.info(`Font ${font.family} loaded from ${font.url}`);
        } else {
          if (this.logging == "VERBOSE" || this.logging == "WARN")
            console.warn(`Unable to load font ${font.family} from ${font.url}`);
        }
      } catch (e) {
        if (this.logging == "VERBOSE" || this.logging == "WARN") {
          console.warn(`Unable to load font ${font.family} from ${font.url}`);
          console.warn(e);
        }
      }
    }
  }
  async getFontUrl(fontUrl) {
    const response = await fetch(fontUrl);
    const blob = await response.blob();
    return URL.createObjectURL(blob);
  }
  async loadFont(font) {
    const url = await this.getFontUrl(font.url);
    const fontFace = new FontFace(font.family, `url(${url})`, font.descriptors || {});
    const loadedFace = await fontFace.load();
    document.fonts.add(loadedFace);
    return fontFace.status === "loaded";
  }
};
export {
  ASS as default
};
